import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;

/**
 * Copyright (C) 2011 Keripo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author Keripo
 * @version 2011/09/05
 */

public class DNCostumeRenamer {
	
	private static String MATCH_PATTERN = null;
	private static String RENAME_PATTERN = null;
	private static String SRC_DIR = null;
	private static String SRC_DIR_2 = null;
	private static String DEST_DIR = null;
	private static ArrayList<String> RENAME_LIST = null;
	
	private static void copy(String src_path, String dest_path) throws Exception {
		FileReader in = new FileReader(src_path);
		FileWriter out = new FileWriter(dest_path);
		
		int c;
		while ((c = in.read()) != -1) {
			out.write(c);
		}
		
		in.close();
		out.close();
	}
	
	private static void check(File f) {
		if (f.exists() && f.canRead()) {
			if (f.isDirectory()) {
				for (File file : f.listFiles()) {
					check(file);
				}
			} else {
				String name = f.getName();
				if (name.matches(MATCH_PATTERN)) {
					try {
						if (name.indexOf('.') != -1) {
							name = name.substring(0, name.lastIndexOf('.')); // Remove file extension
						}
						File output_dir = new File(DEST_DIR + File.separatorChar + name);
						output_dir.mkdirs();
						for (String rename : RENAME_LIST) {
							copy(f.getPath(), output_dir.getPath() + File.separatorChar + rename);
						}
						System.out.println("Renamed: " + name);
					} catch (Exception e) {
						System.out.println("Failed to rename: " + name);
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	private static void add(File f) {
		if (f.exists() && f.canRead()) {
			if (f.isDirectory()) {
				for (File file : f.listFiles()) {
					add(file);
				}
			} else {
				String name = f.getName();
				if (name.matches(RENAME_PATTERN) && !RENAME_LIST.contains(name)) {
					RENAME_LIST.add(name);
				}
			}
		}
	}
	
	private static void print_usage() {
		String program = new DNCostumeRenamer().getClass().getName();
		System.out.println(
				"=====================================================\n" +
				"Dragon Nest Batch Costume Renamer v2\n" +
				"\n" +
				"Usage:\n" +
				"\tjava -jar " + program + " <match_pattern> <rename_pattern> <src_dir> (<src_dir_2>) <dest_dir>\n" +
				"\n" +
				"For example:\n" +
				"\tjava -jar " +
						program +
						" \"so_\\w*_body.skn\"" +
						" \"so_(normal01|normal02|normal03)_body.skn\"" +
						" \".\\Resources\\resource\\char\\player\\sorceress\\parts\"" +
						" \".\\Parts\\so_normal\"" +
						"\n" +
				"\n" +
				"See http://download.oracle.com/javase/6/docs/api/java/util/regex/Pattern.html\n" +
				"for more info on how to write regex patterns.\n" +
				"Optionally use <src_dir_2> for cash shop items.\n" +
				"\n" +
				"~Keripo\n" +
				"====================================================="
				);
		System.exit(1);
	}
	
	private static void print_started() {
		System.out.println(
				"=====================================================\n" +
				"Dragon Nest Batch Costume Renamer v2\n" +
				"Last updated: 2011/09/05\n" +
				"~Keripo\n" +
				"\n" +
				"<match_pattern> = " + MATCH_PATTERN + "\n" +
				"<rename_pattern> = " + RENAME_PATTERN + "\n" +
				"<src_dir> = " + SRC_DIR + "\n" +
				"<src_dir2> = " + SRC_DIR_2 + "\n" +
				"<dest_dir> = " + DEST_DIR + "\n" +
				"====================================================="
				);				
	}
	
	private static void print_finished() {
		System.out.println(
				"=====================================================\n" +
				"Finished renaming files in: " + SRC_DIR + "\n" +
				"and in: " + SRC_DIR_2 + "\n" +
				"Use pakpack.exe to pack the folders in: " + DEST_DIR + "\n" +
				"\n" +
				"Happy modding!\n" +
				"\n" +
				"~Keripo\n" +
				"====================================================="
				);
	}
	
	public static void main(String[] args) {
		File src_dir = null;
		File src_dir_2 = null;
		
		// Args
		if (args.length == 5) {
			MATCH_PATTERN = args[0];
			RENAME_PATTERN = args[1];
			SRC_DIR = args[2];
			SRC_DIR_2 = args[3];
			DEST_DIR = args[4];
		} else if (args.length == 4) {
			MATCH_PATTERN = args[0];
			RENAME_PATTERN = args[1];
			SRC_DIR = args[2];
			DEST_DIR = args[3];
		} else {
			print_usage();
		}
		
		RENAME_LIST = new ArrayList<String>();
		src_dir = new File(SRC_DIR);
		add(src_dir);
		if (SRC_DIR_2 != null) {
			src_dir_2 = new File(SRC_DIR_2);
			add(src_dir_2);
		}
		
		print_started();
		
		// Search and rename
		check(src_dir);
		if (SRC_DIR_2 != null) {
			check(src_dir_2);
		}
		
		// Done
		print_finished();
	}

}
